import { Base } from "./Base";
import { User } from "./UserService";
import type { Team, TeamEntity } from "./TeamService";
import type { K8sCluster } from "./K8sCluster";
import type { Invitation, InvitationEntity } from "./InvitationService";
import type { BillingPlan } from "./BillingPlan";
import type { InvitationDomain, InvitationDomainEntity } from "./InvitationDomain";
import type { MapToEntity } from "./types/types";
import type { Except } from "type-fest";
export declare const spaceKinds: readonly ["Personal", "Team"];
export type SpaceKind = typeof spaceKinds[number];
export declare const spaceFeatures: readonly ["DevCluster"];
export type SpaceFeature = typeof spaceFeatures[number];
/**
 *
 * @remarks
 * This interface should be generated using OpenAPI generator in the future.
 *
 * @alpha
 */
export interface Space {
    id?: string;
    name: string;
    description?: string;
    displayName?: string;
    website?: string;
    createdById?: string;
    createdAt?: string;
    updatedAt?: string;
    kind?: SpaceKind;
    features?: SpaceFeature[];
    users?: User[];
    teams?: Team[];
    invitations?: Invitation[];
    invitationDomains?: InvitationDomain[];
}
export type SpaceEntity = Except<MapToEntity<Space>, "teams" | "invitations" | "invitationDomains"> & {
    teams?: TeamEntity[];
    invitations?: InvitationEntity[];
    invitationDomains?: InvitationDomainEntity[];
};
/**
 *
 * The class for consuming all `space` resources.
 *
 * @remarks
 * This class should be generated using OpenAPI generator in the future.
 *
 * @alpha
 */
declare class SpaceService extends Base {
    /**
     * Get one space by space name
     */
    getOne({ name, queryString }: {
        name: string;
        queryString?: string;
    }): Promise<Space>;
    /**
     * Get spaces
     */
    getMany(queryString?: string): Promise<Space[]>;
    /**
     * Create one space
     */
    createOne(space: Space): Promise<Space>;
    /**
     * Add feature to users' Personal Spaces.
     * @param feature - Feature to add
     * @param users - Array of usernames or email addresses
     */
    addSpaceFeature(feature: SpaceFeature, users: string[]): Promise<Record<string, unknown>>;
    /**
     * Remove feature from users' Personal Spaces.
     * @param feature - Feature remove add
     * @param users - Array of usernames or email addresses
     */
    removeSpaceFeature(feature: SpaceFeature, users: string[]): Promise<Record<string, unknown>>;
    /**
     * Update one space
     */
    updateOne(spaceName: string, space: Space): Promise<Space>;
    /**
     * Delete one space by space name
     */
    deleteOne({ name }: {
        name: string;
    }): Promise<void>;
    /**
     * Get bored-secret by space name and cluster id
     */
    getBoredSecret({ name, clusterId, }: {
        name: string;
        clusterId: string;
    }): Promise<{
        token: string;
    }>;
    /**
     * Get cluster token by space name and cluster id
     */
    getClusterToken({ name, clusterId, }: {
        name: string;
        clusterId: string;
    }): Promise<{
        token: string;
    }>;
    /**
     * Get all clusters in one space by space name
     */
    getClusters({ name }: {
        name: string;
    }): Promise<K8sCluster[]>;
    /**
     * Get all invitation domains in one space by space name
     */
    getInvitationDomains({ name }: {
        name: string;
    }): Promise<InvitationDomain[]>;
    /**
     * Add one invitation domain in a Space by space name
     */
    addInvitationDomain({ name, domain, }: {
        name: Space["name"];
        domain: InvitationDomain["domain"];
    }): Promise<InvitationDomain>;
    /**
     * Delete one invitation domain in a Space by space name and invitation domain id
     */
    deleteInvitationDomain({ name, invitationDomainId, }: {
        name: Space["name"];
        invitationDomainId: InvitationDomain["id"];
    }): Promise<void>;
    /**
     * Get one cluster by cluster id in one space by space name
     */
    getOneCluster({ clusterId, name, }: {
        clusterId: string;
        name: string;
    }): Promise<K8sCluster>;
    /**
     * Create a cluster in one space by space name
     */
    createOneCluster({ cluster, name, }: {
        cluster: K8sCluster;
        name: string;
    }): Promise<K8sCluster>;
    /**
     * Update a cluster
     */
    updateOneCluster({ cluster }: {
        cluster: K8sCluster;
    }): Promise<K8sCluster>;
    /**
     * Replace a cluster
     */
    replaceOneCluster({ cluster }: {
        cluster: K8sCluster;
    }): Promise<K8sCluster>;
    /**
     * Delete a cluster by cluster id in one space by space name
     */
    deleteOneCluster({ clusterId, name }: {
        clusterId: string;
        name: string;
    }): Promise<void>;
    /**
     * Remove one user by username from a space by space name
     * @throws SpaceNotFoundException, UserNameNotFoundException, ForbiddenException, CantRemoveOwnerFromSpaceException
     */
    removeOneUser({ username, name }: {
        username: string;
        name: string;
    }): Promise<void>;
    /**
     * Get billing plan of space by space name
     */
    getBillingPlan({ name }: {
        name: string;
    }): Promise<BillingPlan>;
    /**
     * Downgrades the billing plan of space by space name
     */
    downgradeBillingPlan({ name }: {
        name: string;
    }): Promise<BillingPlan>;
}
export { SpaceService };
