import { isDevCluster } from "./K8sCluster";
export var Roles;
(function (Roles) {
    Roles["Admin"] = "Admin";
    Roles["Owner"] = "Owner";
    Roles["Member"] = "Member";
    Roles["None"] = "None";
})(Roles || (Roles = {}));
/**
 * Space Actions
 */
export var Actions;
(function (Actions) {
    Actions[Actions["DeleteSpace"] = 0] = "DeleteSpace";
    Actions[Actions["PatchSpace"] = 1] = "PatchSpace";
    Actions[Actions["RenameSpace"] = 2] = "RenameSpace";
    Actions[Actions["CreateInvitation"] = 3] = "CreateInvitation";
    Actions[Actions["PatchInvitation"] = 4] = "PatchInvitation";
    Actions[Actions["RevokeInvitation"] = 5] = "RevokeInvitation";
    Actions[Actions["AddInvitationDomain"] = 6] = "AddInvitationDomain";
    Actions[Actions["DeleteInvitationDomain"] = 7] = "DeleteInvitationDomain";
    Actions[Actions["CreateTeam"] = 8] = "CreateTeam";
    Actions[Actions["DeleteTeam"] = 9] = "DeleteTeam";
    Actions[Actions["PatchTeam"] = 10] = "PatchTeam";
    Actions[Actions["GetBillingPageToken"] = 11] = "GetBillingPageToken";
    Actions[Actions["ChangeSpacePlan"] = 12] = "ChangeSpacePlan";
})(Actions || (Actions = {}));
export var K8sClusterActions;
(function (K8sClusterActions) {
    // Check if user can access the K8sCluster assuming the Space is accessible
    K8sClusterActions[K8sClusterActions["AccessK8sCluster"] = 0] = "AccessK8sCluster";
    K8sClusterActions[K8sClusterActions["PatchK8sCluster"] = 1] = "PatchK8sCluster";
    K8sClusterActions[K8sClusterActions["DeleteK8sCluster"] = 2] = "DeleteK8sCluster";
})(K8sClusterActions || (K8sClusterActions = {}));
export var TeamActions;
(function (TeamActions) {
    TeamActions[TeamActions["AddUser"] = 0] = "AddUser";
    TeamActions[TeamActions["RemoveUser"] = 1] = "RemoveUser";
})(TeamActions || (TeamActions = {}));
export class Permissions {
    constructor() {
        this.getOwnerTeams = (space) => {
            var _a;
            const teams = (_a = space.teams) !== null && _a !== void 0 ? _a : [];
            return teams.filter(({ kind }) => kind === "Owner");
        };
        this.getAdminTeams = (space) => {
            var _a;
            const teams = (_a = space.teams) !== null && _a !== void 0 ? _a : [];
            return teams.filter(({ kind }) => kind === "Admin");
        };
        this.getNormalTeams = (space) => {
            var _a;
            const teams = (_a = space.teams) !== null && _a !== void 0 ? _a : [];
            return teams.filter(({ kind }) => kind === "Normal");
        };
        this.isUserInTeam = (team, userId) => {
            var _a;
            if (!((_a = team.users) === null || _a === void 0 ? void 0 : _a.length)) {
                return false;
            }
            return Boolean(team.users.find((u) => u.id === userId));
        };
        this.canPatchOrRevokeInvitation = (forRevokeInvitation, role) => {
            var _a;
            const isAdminOrOwner = [Roles.Owner, Roles.Admin].includes(role);
            if (isAdminOrOwner) {
                return true;
            }
            if (
            // If there is an invitationId to be revoked
            (forRevokeInvitation === null || forRevokeInvitation === void 0 ? void 0 : forRevokeInvitation.invitationId) &&
                // If this user has created more than one invitation
                ((_a = forRevokeInvitation === null || forRevokeInvitation === void 0 ? void 0 : forRevokeInvitation.invitationIdsCreatedByUserId) === null || _a === void 0 ? void 0 : _a.length) > 0 &&
                (
                // If invitation to revoke was created by userId
                forRevokeInvitation === null || forRevokeInvitation === void 0 ? void 0 : forRevokeInvitation.invitationIdsCreatedByUserId.find((invitationIdCreatedByUserId) => invitationIdCreatedByUserId === (forRevokeInvitation === null || forRevokeInvitation === void 0 ? void 0 : forRevokeInvitation.invitationId)))) {
                return true;
            }
            return false;
        };
    }
    /**
     * Clarifies whether a given user can perform an action in a given space.
     * @param action - `Actions` enum value
     * @param forSpace - Space object that must contain `{ teams: Team[] }`
     * @param forUserId - string userId
     * @param forRevokeInvitation - Additional information to determine if a user can revoke invitation
     * @returns boolean
     * @throws "Could not get role for space with no teams" exception
     */
    canSpace(action, forSpace, forUserId, forRevokeInvitation) {
        const role = this.getRole(forSpace, forUserId);
        const isAdminOrOwner = [Roles.Owner, Roles.Admin].includes(role);
        switch (action) {
            case Actions.ChangeSpacePlan:
                return role === Roles.Owner;
            case Actions.DeleteSpace:
                return forSpace.kind !== "Personal" && role === Roles.Owner;
            case Actions.PatchInvitation:
                return this.canPatchOrRevokeInvitation(forRevokeInvitation, role);
            case Actions.RevokeInvitation:
                return this.canPatchOrRevokeInvitation(forRevokeInvitation, role);
            case Actions.RenameSpace:
                return forSpace.kind !== "Personal" && isAdminOrOwner;
            case Actions.CreateInvitation:
                return isAdminOrOwner;
            case Actions.PatchTeam:
                return isAdminOrOwner;
            case Actions.PatchSpace:
                return isAdminOrOwner;
            case Actions.CreateTeam:
                return isAdminOrOwner;
            case Actions.DeleteTeam:
                return isAdminOrOwner;
            case Actions.AddInvitationDomain:
                return isAdminOrOwner;
            case Actions.GetBillingPageToken:
                return isAdminOrOwner;
            case Actions.DeleteInvitationDomain:
                return isAdminOrOwner;
            default:
                return false;
        }
    }
    /**
     * Clarifies whether a given user can execute the specified Team action
     * @param action - 'TeamActions' enum value
     * @param space - Space object that must contain `{ teams: Team[] }`
     * @param team - Team object
     * @param forUserId - User that is executing the action
     * @param targetUserId - User that is the target of the exection, e.g. user to be removed
     * @returns
     */
    // eslint-disable-next-line max-params
    canTeam(action, space, team, forUserId, targetUserId) {
        const role = this.getRole(space, forUserId);
        const isAdminOrOwner = [Roles.Owner, Roles.Admin].includes(role);
        switch (action) {
            case TeamActions.AddUser: {
                if (!this.canSpace(Actions.PatchTeam, space, forUserId)) {
                    return false;
                }
                // Prevent adding any user to Owner team of a Personal Space
                if (team.kind === "Owner" && space.kind === "Personal") {
                    return false;
                }
                if (team.kind === "Owner") {
                    return role === Roles.Owner;
                }
                if (team.kind === "Admin") {
                    return isAdminOrOwner;
                }
                if (team.kind === "Normal") {
                    return isAdminOrOwner;
                }
                return false;
            }
            case TeamActions.RemoveUser: {
                if (!this.canSpace(Actions.PatchTeam, space, forUserId)) {
                    return false;
                }
                // Prevent removing creator of Personal Space from the Owner team
                if (team.kind === "Owner" &&
                    space.kind === "Personal" &&
                    space.createdById === targetUserId) {
                    return false;
                }
                if (team.kind === "Owner") {
                    return role === Roles.Owner;
                }
                if (team.kind === "Admin") {
                    return isAdminOrOwner;
                }
                if (team.kind === "Normal") {
                    return isAdminOrOwner;
                }
                return false;
            }
            default: {
                return false;
            }
        }
    }
    /**
     * Clarifies whether a given user can perform an action for a given K8sCluster.
     * @param action - `Actions` enum value
     * @param forSpace - Space object that must contain `{ teams: Team[] }`
     * @param forK8sCluster - K8sCluster
     * @param forUserId - string userId
     * @returns boolean
     * @throws "Could not get role for space with no teams" exception
     */
    canK8sCluster(action, forSpace, forK8sCluster, forUserId) {
        let canI = false;
        const isOwnerAdmin = [Roles.Owner, Roles.Admin].includes(this.getRole(forSpace, forUserId));
        switch (action) {
            // Check if user can access the K8sCluster assuming the Space is accessible
            // DevCluster K8sClusters can only be accessed by the creator or Admin/Owner
            case K8sClusterActions.AccessK8sCluster:
                canI =
                    !isDevCluster(forK8sCluster) || isOwnerAdmin || forK8sCluster.createdById === forUserId;
                break;
            // Check if user can patch the K8sCluster assuming the Space is accessible
            // DevCluster K8sClusters can only be patched by the creator or Admin/Owner
            case K8sClusterActions.PatchK8sCluster:
                canI =
                    !isDevCluster(forK8sCluster) || isOwnerAdmin || forK8sCluster.createdById === forUserId;
                break;
            // Admin, Owner or K8sCluster creator can delete it
            case K8sClusterActions.DeleteK8sCluster: {
                canI = isOwnerAdmin || forK8sCluster.createdById === forUserId;
                break;
            }
            default:
                throw new Error(`Unknown action ${action}`);
        }
        return canI;
    }
    /**
     * DEPRECATED. WILL BE REMOVED.
     * Clarifies whether a given user can perform an action in a given space.
     * @param action - `Actions` enum value
     * @param forSpace - Space object that must contain `{ teams: Team[] }`
     * @param forUserId - string userId
     * @returns boolean
     * @throws "Could not get role for space with no teams" exception
     * @deprecated Use .canSpace instead.
     */
    canI(action, forSpace, forUserId) {
        return this.canSpace(action, forSpace, forUserId);
    }
    /**
     * Gets a role the user with specified user Id has in specified `space`.
     * @param space - Space object that must contain `{ teams: Team[] }`
     * @param forUserId - string userId
     * @returns Role enum value
     * @throws "Could not get role for space with no teams" exception
     */
    getRole(space, forUserId) {
        var _a;
        if (!space.teams) {
            throw new Error("Could not get role for space with no teams");
        }
        if (this.getOwnerTeams(space).filter((team) => this.isUserInTeam(team, forUserId)).length) {
            return Roles.Owner;
        }
        if (this.getAdminTeams(space).filter((team) => this.isUserInTeam(team, forUserId)).length) {
            return Roles.Admin;
        }
        if ((_a = space.users) === null || _a === void 0 ? void 0 : _a.map((user) => user.id).includes(forUserId)) {
            return Roles.Member;
        }
        return Roles.None;
    }
}
