import { LensSDKException, ForbiddenException, UnauthorizedException, BadRequestException, } from "./common.exceptions";
import axios from "axios";
const parseHTTPErrorCode = (exception) => {
    var _a, _b;
    if ((exception === null || exception === void 0 ? void 0 : exception.name) && (exception === null || exception === void 0 ? void 0 : exception.message) && ((_a = exception === null || exception === void 0 ? void 0 : exception.message) === null || _a === void 0 ? void 0 : _a.length)) {
        const execArr = (_b = /code\s([\d]{3})/g.exec(exception.message)) !== null && _b !== void 0 ? _b : [];
        if ((execArr === null || execArr === void 0 ? void 0 : execArr.length) >= 2) {
            return parseInt(execArr[1], 10);
        }
    }
    return null;
};
/**
 * Transforms an axios error response (with .data key) into a response with .body key
 * (for backwards compatibility)
 */
const toPlatformErrorResponse = async (response) => {
    if (response === null || response === void 0 ? void 0 : response.data) {
        return {
            ...response,
            body: response === null || response === void 0 ? void 0 : response.data,
        };
    }
    return undefined;
};
/**
 * Executes a given function, catching all exceptions. When an exception is caught
 * it is converted to strongly-typed `LensPlatformExtension` and thrown again.
 * @param fn - a function
 * @param exceptionsMap - map of HTTP error codes onto expected exception creators
 * @returns the ressult of `fn`
 * @throws extected exceptions or `LensPlatformException` with code 400 if it caught something unexpected
 * @example
 * ```
 * const json = throwExpected(
 *  () => fetch.get(url),
 *    {
 *      404: e => e.url.includes("/user") ?
 *        new NotFoundException(`User ${username} not found`) :
 *        new NotFoundException(`Something else not found`),
 *      500: () => new TokenNotFoundException()
 *    }
 * );
 * ```
 */
export const throwExpected = async (fn, exceptionsMap = {}) => {
    var _a, _b, _c, _d;
    try {
        const result = await fn();
        return result;
    }
    catch (error) {
        if (axios.isAxiosError(error)) {
            const httpStatusCode = (_b = (_a = error.response) === null || _a === void 0 ? void 0 : _a.status) !== null && _b !== void 0 ? _b : parseHTTPErrorCode(error);
            if (!httpStatusCode) {
                throw new LensSDKException(httpStatusCode, "Unexpected exception [Lens Platform SDK]", error);
            }
            const mappedExceptionFn = exceptionsMap[httpStatusCode];
            const data = (_c = error.response) === null || _c === void 0 ? void 0 : _c.data;
            const message = data === null || data === void 0 ? void 0 : data.message;
            if (mappedExceptionFn) {
                throw mappedExceptionFn(await toPlatformErrorResponse(error === null || error === void 0 ? void 0 : error.response));
            }
            else if (httpStatusCode === 400) {
                throw new BadRequestException(message, error);
            }
            else if (httpStatusCode === 401) {
                throw new UnauthorizedException(message, error);
            }
            else if (httpStatusCode === 403) {
                throw new ForbiddenException(message, error);
            }
        }
        // If axios.isAxiosError(error) returns falsy
        throw new LensSDKException((_d = error === null || error === void 0 ? void 0 : error.response) === null || _d === void 0 ? void 0 : _d.status, "Unexpected exception [Lens Platform SDK]", error);
    }
};
